@@ -2,7 +2,7 @@
 /**
  * This file is part of BibORB
  *
- * Copyright (C) 2003-2004  Guillaume Gardey
+ * Copyright (C) 2003-2004  Guillaume Gardey (ggardey@club-internet.fr)
  * 
  * BibORB is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -40,16 +40,41 @@
  */
 
 require_once("config.php");     // load configuration variables
-require_once("functions.php");  // load needed functions
-require_once("biborbdb.php");   // load biborb database
-require_once("interface.php");  // load function to generate the interface
+require_once("php/functions.php");  // load needed functions
+require_once("php/biborbdb.php");   // load biborb database
+require_once("php/interface.php");  // load function to generate the interface
+require_once("php/auth.php");       // load authentication class
 
 /**
  * Load the session
  */
+session_cache_limiter('nocache');
 session_name($session_id);
 session_start();
 
+/**
+
+ */
+if(get_magic_quotes_gpc()) {
+    $_POST = array_map('stripslashes_deep', $_POST);
+    $_GET = array_map('stripslashes_deep', $_GET);
+    $_COOKIE = array_map('stripslashes_deep', $_COOKIE);
+}
+
+/**
+ *  i18n
+ */
+if(!array_key_exists('language',$_GET)){
+    if(!array_key_exists('language',$_SESSION) || !$GLOBALS['display_language_selection']){
+        $_SESSION['language'] = $GLOBALS['language'];
+    }
+}
+else{
+    $_SESSION['language'] = $_GET['language'];
+}
+load_i18n_config($_SESSION['language']);
+
+
 /**
  * To store an error or a message. (mode=result)
  */
@@ -73,25 +98,36 @@ else{
  *  admin => may modify, create or delete
  *  user => only for read purpose
  */
+
 if(!$disable_authentication){
-  if(!array_key_exists('usermode',$_SESSION)){
-      $_SESSION['usermode'] = "user";
-  }
+    if(!array_key_exists('auth',$_SESSION)){
+        $_SESSION['auth'] = new Auth();
+    }
+        
+    if(!array_key_exists('user',$_SESSION)){
+        $_SESSION['user_is_admin'] = FALSE;
+    }
 }
 else{
-  $_SESSION['usermode'] = "admin";
+    $_SESSION['user_is_admin'] = TRUE;
 }
 
+
+
 /*
     Look in $_GET for an action to be performed
  */
 if(isset($_GET['action'])){
     switch($_GET['action']){
-        
+        case 'select_lang':
+            $_SESSION['language'] = $_GET['lang'];
+            load_i18n_config($_SESSION['language']);
+            break;
+            
         /*
             Create a database
          */
-        case 'create':
+        case msg("Create"):
             $error_or_message = create_database($_GET['database_name'],
                                                 $_GET['description']);
             break;
@@ -99,15 +135,16 @@ if(isset($_GET['action'])){
         /*
             Delete a database
          */
-        case 'delete':
+        case msg("Delete"):
             $error_or_message['message'] = delete_database($_GET['database_name']);
             break;
         
         /*
             Logout
          */
-        case 'logout':
-            $_SESSION['usermode'] = "user";
+        case "logout":
+            unset($_SESSION['user']);
+            $_SESSION['user_is_admin'] = FALSE;
             break;
             
         default:
@@ -128,17 +165,17 @@ if(isset($_POST['action'])){
             $login = $_POST['login'];
             $mdp = $_POST['mdp'];
             if($login=="" || $mdp==""){
-                $error_or_message['error'] = "You must fill both login and password!";
+                $error_or_message['error'] = msg("LOGIN_MISSING_VALUES");
                 $mode = "login";
             }
             else {
-                $loggedin = check_login($login,$mdp);
+                $loggedin = $_SESSION['auth']->is_valid_user($login,$mdp);
                 if($loggedin){
                     $_SESSION['user'] = $login;
-                    $_SESSION['usermode'] = "admin";
+                    $_SESSION['user_is_admin'] = $_SESSION['auth']->is_admin_user($_SESSION['user']);
                 }
                 else {
-                    $error_or_message['error'] = "Wrong login or password";
+                    $error_or_message['error'] = msg("LOGIN_WRONG_USERNAME_PASSWORD");
                     $mode = "login";
                 }
             }
